# Generated by Django 4.0.1 on 2022-08-19 15:49

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Thing",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=255)),
                ("big", models.BooleanField(default=False)),
                ("clever", models.BooleanField(default=False)),
                (
                    "element_type",
                    models.CharField(
                        choices=[("e", "Earth"), ("w", "Water"), ("a", "Air"), ("f", "Fire")], max_length=1
                    ),
                ),
                ("category", models.IntegerField(choices=[(0, "Magma"), (1, "Quasigroup"), (2, "Monoid")], default=0)),
                ("count", models.IntegerField(default=0)),
                ("description", models.TextField(blank=True)),
                ("notes_file", models.FileField(blank=True, upload_to="django_functest/tests/uploads")),
            ],
        ),
    ]
